@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2023 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    MSTLLogicControl.h
 /// @author  Daniel Krajzewicz
 /// @author  Julia Ringel
@@ -6,40 +18,27 @@
 /// @author  Michael Behrisch
 /// @author  Friedemann Wesner
 /// @date    Sept 2002
-/// @version $Id: MSTLLogicControl.h 15218 2013-11-26 21:09:51Z behrisch $
 ///
 // A class that stores and controls tls and switching of their programs
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
-// Copyright (C) 2001-2013 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-#ifndef MSTLLogicControl_h
-#define MSTLLogicControl_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <vector>
 #include <map>
-#include "MSTrafficLightLogic.h"
-#include "MSSimpleTrafficLightLogic.h"
 #include <utils/common/Command.h>
-#include <utils/common/UtilExceptions.h>
+#include <utils/common/StdDefs.h>
+#include <utils/xml/SUMOXMLDefinitions.h>
+
+
+// ===========================================================================
+// class declarations
+// ===========================================================================
+class MSTrafficLightLogic;
+class MSLink;
+class MSLane;
+class MSPhaseDefinition;
+class OutputDevice;
 
 
 // ===========================================================================
@@ -88,21 +87,18 @@ public:
         /// @brief Constructor
         TLSLogicVariants();
 
-
         /// @brief Destructor
         ~TLSLogicVariants();
 
-
         /** @brief Verifies traffic lights loaded from the network
          *
-         * Comparse the phase state sizes of each phase with the accoring tls' number
+         * Compare the phase state sizes of each phase with the according tls' number
          *  of controlled links.
          * @return Whether all tls programs are valid
          */
         bool checkOriginalTLS() const;
 
-
-        /** @brief Adds a logic (program)
+        /** @brief Adds a logic (program). In case of an error the logic gets deleted.
          *
          * @param[in] programID The sub-id of this program
          * @param[in] logic The logic to add
@@ -112,14 +108,17 @@ public:
         bool addLogic(const std::string& programID, MSTrafficLightLogic* logic, bool netWasLoaded,
                       bool isNewDefault = true);
 
-
-
         MSTrafficLightLogic* getLogic(const std::string& programID) const;
         void addSwitchCommand(OnSwitchAction* c);
         std::vector<MSTrafficLightLogic*> getAllLogics() const;
         void saveInitialStates();
+        void saveState(OutputDevice& out);
         bool isActive(const MSTrafficLightLogic* tl) const;
         MSTrafficLightLogic* getActive() const;
+
+        /// @brief return the default program (that last used program except TRACI_PROGRAM)
+        MSTrafficLightLogic* getDefault() const;
+
         void switchTo(MSTLLogicControl& tlc, const std::string& programID);
 
         /* @brief get logic by programID. For the special case "off"
@@ -134,14 +133,16 @@ public:
 
 
         void executeOnSwitchActions() const;
-        void addLink(MSLink* link, MSLane* lane, unsigned int pos);
-
-
+        void addLink(MSLink* link, MSLane* lane, int pos);
+        void ignoreLinkIndex(int pos);
 
     private:
         /// @brief The currently used program
         MSTrafficLightLogic* myCurrentProgram;
 
+        /// @brief The program that would be used in the absence of TraCI
+        MSTrafficLightLogic* myDefaultProgram;
+
         /// @brief A map of subkeys to programs
         std::map<std::string, MSTrafficLightLogic*> myVariants;
 
@@ -151,7 +152,6 @@ public:
         /// @brief The list of actions/commands to execute on switch
         std::vector<OnSwitchAction*> mySwitchActions;
 
-
     private:
         /// @brief Invalidated copy constructor.
         TLSLogicVariants(const TLSLogicVariants&);
@@ -163,15 +163,12 @@ public:
     };
 
 
-
     /// @brief Constructor
     MSTLLogicControl();
 
-
     /// @brief Destructor
     ~MSTLLogicControl();
 
-
     /** @brief Lets MSTLLogicControl know that the network has been loaded
      *
      * This method must be called after the network (including the initial tls
@@ -191,7 +188,6 @@ public:
      */
     bool closeNetworkReading();
 
-
     /** @brief Lets all running (current) tls programs apply their current signal states to links they control
      * @param[in] t The current time
      * @see MSTrafficLightLogic::setTrafficLightSignals
@@ -200,7 +196,6 @@ public:
      */
     void setTrafficLightSignals(SUMOTime t) const;
 
-
     /** @brief Returns a vector which contains all logics
      *
      * All logics are included, active (current) and non-active
@@ -208,7 +203,6 @@ public:
      */
     std::vector<MSTrafficLightLogic*> getAllLogics() const;
 
-
     /** @brief Returns the variants of a named tls
      *
      * @param[in] id The id of the tls to get variants of
@@ -217,7 +211,6 @@ public:
      */
     TLSLogicVariants& get(const std::string& id) const;
 
-
     /** @brief Returns a single program (variant) defined by the tls id and the program programID
      *
      * @param[in] id The id of the tls to get program of
@@ -226,7 +219,6 @@ public:
      */
     MSTrafficLightLogic* get(const std::string& id, const std::string& programID) const;
 
-
     /** @brief Returns the active program of a named tls
      *
      * @param[in] id The id of the tls to get the active program of
@@ -234,7 +226,6 @@ public:
      */
     MSTrafficLightLogic* getActive(const std::string& id) const;
 
-
     /**
      * Returns the ids of all existing variants-structures, wich are the ids of their
      * contained tls logics (not the logic's programm-ids)
@@ -242,7 +233,6 @@ public:
      */
     std::vector<std::string> getAllTLIds() const;
 
-
     /** @brief Adds a tls program to the container
      *
      * If a tls with the given id is not yet known, a TLSLogicVariants structure
@@ -252,7 +242,7 @@ public:
      * If the tls to add is loaded from an additional file (indicated by myNetWasLoaded,
      *  see closeNetworkReading), links from previously loaded tls are adapted to the logic.
      *  This may throw a ProcessError in the case no tls program was loaded for this
-     *  tls before (was not defined in the network).
+     *  tls before (was not defined in the network). In case of an error the logic gets deleted.
      *
      * The parameter newDefault defines whether this program will be used as the new
      *  default program of this tls. This means that an existing tls program for this
@@ -267,22 +257,18 @@ public:
     bool add(const std::string& id, const std::string& programID,
              MSTrafficLightLogic* logic, bool newDefault = true);
 
-
-
     /** @brief Returns the information whether the named tls is stored
      * @param[in] id The id of the tls to ask for
      * @return Whether a tls with the given id is known
      */
     bool knows(const std::string& id) const;
 
-
     /** @brief Returns whether the given tls program is the currently active for his tls
      * @param[in] tl The tls to ask for
      * @return Whether the given tl is currently active (or a different program is used)
      */
     bool isActive(const MSTrafficLightLogic* tl) const;
 
-
     /** @brief Switches the named (id) tls to the named (programID) program
      *
      * The program with the used programID must be previously added.
@@ -293,8 +279,6 @@ public:
      */
     void switchTo(const std::string& id, const std::string& programID);
 
-
-
     /// @name WAUT definition methods
     /// @{
 
@@ -304,10 +288,11 @@ public:
      * @param[in] refTime The reference time of the WAUT
      * @param[in] id The ID of the WAUT
      * @param[in] startProg The begin program of the WAUT
+     * @param[in] period The period with which to repeat the switches
      * @exception InvalidArgument If the id is already used by another WAUT
      */
     void addWAUT(SUMOTime refTime, const std::string& id,
-                 const std::string& startProg);
+                 const std::string& startProg, SUMOTime period);
 
 
     /** @brief Adds a WAUT switch step to a previously built WAUT
@@ -368,6 +353,16 @@ public:
      */
     std::pair<SUMOTime, MSPhaseDefinition> getPhaseDef(const std::string& tlid) const;
 
+    /// @brief switch all logic variants to 'off'
+    void switchOffAll();
+
+    /** @brief Saves the current tls states into the given stream
+     */
+    void saveState(OutputDevice& out);
+
+    /** @brief Clear all tls states before quick-loading state */
+    void clearState(SUMOTime time, bool quickReload = false);
+
 
 
 protected:
@@ -385,7 +380,7 @@ protected:
          * @param[in] wautid The id of the WAUT
          * @param[in] index The first position within the WAUT table
          */
-        SwitchInitCommand(MSTLLogicControl& p, const std::string& wautid, unsigned int index)
+        SwitchInitCommand(MSTLLogicControl& p, const std::string& wautid, int index)
             : myParent(p), myWAUTID(wautid), myIndex(index) { }
 
 
@@ -429,7 +424,7 @@ protected:
         /** @brief Returns a reference to the index
          * @return A reference to the index
          */
-        unsigned int& getIndex() {
+        int& getIndex() {
             return myIndex;
         }
 
@@ -442,7 +437,7 @@ protected:
         std::string myWAUTID;
 
         /// @brief The current index within the WAUT switch table
-        unsigned int myIndex;
+        int myIndex;
 
 
     private:
@@ -501,6 +496,8 @@ protected:
         std::string startProg;
         /// @brief The reference time (offset to the switch times)
         SUMOTime refTime;
+        /// @brief The period with which to repeat switches
+        SUMOTime period;
         /// @brief The list of switches to be done by the WAUT
         std::vector<WAUTSwitch> switches;
         /// @brief The list of switches assigned to the WAUT
@@ -534,11 +531,11 @@ protected:
          * @param[in] step The current simulation step
          * @return If a switch shall be done, this method should return true.
          */
-        virtual bool trySwitch(SUMOTime step) = 0;
+        virtual bool trySwitch(SUMOTime step);
 
 
     protected:
-        /** @brief Checks, whether the position of a signal programm is at the GSP ("GuenstigerUmschaltPunkt")
+        /** @brief Checks, whether the position of a signal programm is at the GSP ("Good Switching Point")
          *
          * The GSP must be given as a logic's parameter ("GSP"). Not the simulation second,
          *  but the phase the GSP lies within is used. If the phase the GSP lies within is
@@ -550,7 +547,6 @@ protected:
          */
         bool isPosAtGSP(SUMOTime currentTime, const MSTrafficLightLogic& logic);
 
-
         /** @brief Returns the difference between a given time and the start of the phase
          * @param[in] logic The logic to consider
          * @param[in] toTime The time to ask for
@@ -558,7 +554,6 @@ protected:
          */
         SUMOTime getDiffToStartOfPhase(MSTrafficLightLogic& logic, SUMOTime toTime);
 
-
         /** @brief switches the given logic directly to the given position
          * @param[in] simStep The current simulation time
          * @param[in] logic The logic to switch
@@ -566,7 +561,6 @@ protected:
          */
         void switchToPos(SUMOTime simStep, MSTrafficLightLogic& logic, SUMOTime toTime);
 
-
         /** @brief Returns the GSP-value
          *
          * The GSP must be given as a logic's parameter ("GSP").
@@ -574,8 +568,16 @@ protected:
          * @return The GSP value; 0 if not given.
          * @see MSTrafficLightLogic::getParameterValue
          */
-        unsigned int getGSPValue(const MSTrafficLightLogic& logic) const;
+        SUMOTime getGSPTime(const MSTrafficLightLogic& logic) const;
 
+        /** @brief Changes the destination program's phase to which the tls was switched
+         *
+         * Default does nothing, implemented only in the subclasses.
+         * @param[in] step The current simulation time
+         */
+        virtual void adaptLogic(SUMOTime step) {
+            UNUSED_PARAMETER(step);
+        }
 
     protected:
         /// @brief The current program of the tls to switch
@@ -621,25 +623,15 @@ protected:
                                        MSTrafficLightLogic* from, MSTrafficLightLogic* to,
                                        bool synchron);
 
-
         /// @brief Destructor
         ~WAUTSwitchProcedure_JustSwitch();
 
-
         /** @brief Determines whether a switch is possible.
          * @param[in] step The current simulation step
          * @return This implementation alsways returns true
          */
         bool trySwitch(SUMOTime step);
 
-
-    private:
-        /// @brief Invalidated copy constructor.
-        WAUTSwitchProcedure_JustSwitch(const WAUTSwitchProcedure_JustSwitch&);
-
-        /// @brief Invalidated assignment operator.
-        WAUTSwitchProcedure_JustSwitch& operator=(const WAUTSwitchProcedure_JustSwitch&);
-
     };
 
 
@@ -664,27 +656,11 @@ protected:
         /// @brief Destructor
         ~WAUTSwitchProcedure_GSP();
 
-
-        /** @brief Determines whether a switch is possible.
-         * @param[in] step The current simulation step
-         * @return If a switch shall be done, this method should return true.
-         */
-        bool trySwitch(SUMOTime step);
-
-
     protected:
         /** @brief Stretches the destination program's phase to which the tls was switched
          */
         void adaptLogic(SUMOTime step);
 
-
-    private:
-        /// @brief Invalidated copy constructor.
-        WAUTSwitchProcedure_GSP(const WAUTSwitchProcedure_GSP&);
-
-        /// @brief Invalidated assignment operator.
-        WAUTSwitchProcedure_GSP& operator=(const WAUTSwitchProcedure_GSP&);
-
     };
 
 
@@ -705,18 +681,9 @@ protected:
                                     MSTrafficLightLogic* from, MSTrafficLightLogic* to,
                                     bool synchron);
 
-
         /// @brief Destructor
         ~WAUTSwitchProcedure_Stretch();
 
-
-        /** @brief Determines whether a switch is possible.
-         * @param[in] step The current simulation step
-         * @return If a switch shall be done, this method should return true.
-         */
-        bool trySwitch(SUMOTime step);
-
-
     protected:
         /** @brief Determines the destination program's changes and applies them
          * @param[in] step The current simulation step
@@ -725,7 +692,6 @@ protected:
          */
         void adaptLogic(SUMOTime step);
 
-
         /** @brief Stretches the logic to synchronize
          * @param[in] step The current simulation step
          * @param[in] startPos The position in the destination program to switch to
@@ -733,7 +699,6 @@ protected:
          */
         void stretchLogic(SUMOTime step, SUMOTime startPos, SUMOTime allStretchTime);
 
-
         /** @brief Cuts the logic to synchronize
          * @param[in] step The current simulation step
          * @param[in] startPos The position in the destination program to switch to
@@ -741,46 +706,22 @@ protected:
          */
         void cutLogic(SUMOTime step, SUMOTime startPos, SUMOTime allCutTime);
 
-
     protected:
-        /** @struct StretchBereichDef
+        /** @struct StretchRange
          * @brief A definition of a stretch - Bereich
          */
-        struct StretchBereichDef {
-            /// @brief The begin of a stretch/cut area (time, in s)
-            SUMOReal begin;
-            /// @brief The end of a stretch/cut area (time, in s)
-            SUMOReal end;
+        struct StretchRange {
+            /// @brief The begin of a stretch/cut area
+            SUMOTime begin;
+            /// @brief The end of a stretch/cut area
+            SUMOTime end;
             /// @brief The weight factor of a stretch/cut area
-            SUMOReal fac;
-
+            double fac;
         };
 
-
-        /** @brief Returns the number of given Stretch-areas for the given program
-         * @param[in] from The tls program to get the number of stretch areas from
-         * @return The number of stretch areas
-         */
-        int getStretchAreaNo(MSTrafficLightLogic* from) const;
-
-
-        /** @brief Returns the numbered Stretch-area for the given program
-         *
-         * The first area has normally the number "1", not "0"!
-         * @param[in] from The tls program to get the named stretch area from
-         * @param[in] index The index (identifier) for the area
-         * @return The definition of the stretch area
-         */
-        StretchBereichDef getStretchBereichDef(MSTrafficLightLogic* from, int index) const;
-
-
-    private:
-        /// @brief Invalidated copy constructor.
-        WAUTSwitchProcedure_Stretch(const WAUTSwitchProcedure_Stretch&);
-
-        /// @brief Invalidated assignment operator.
-        WAUTSwitchProcedure_Stretch& operator=(const WAUTSwitchProcedure_Stretch&);
-
+    protected:
+        /// @brief the given Stretch-areas for the "to" program, this is 0-based indexed, while the input is 1-based
+        std::vector<StretchRange> myStretchRanges;
     };
 
 
@@ -821,9 +762,3 @@ private:
     MSTLLogicControl& operator=(const MSTLLogicControl&);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
