@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2022 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2023 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -32,6 +32,7 @@
 #include <utils/common/ToString.h>
 #include <utils/iodevices/OutputDevice.h>
 #include "NBNode.h"
+#include "NBAlgorithms.h"
 #include "NBNodeShapeComputer.h"
 
 //#define DEBUG_NODE_SHAPE
@@ -40,7 +41,6 @@
 #define DEBUGCOND (myNode.getID() == "C")
 
 
-#define EXT 100.0
 #define EXT2 10.0
 
 // foot and bicycle paths as well as pure service roads should not get large junctions
@@ -53,7 +53,13 @@ const SVCPermissions NBNodeShapeComputer::SVC_LARGE_TURN(
 // ===========================================================================
 NBNodeShapeComputer::NBNodeShapeComputer(const NBNode& node) :
     myNode(node),
-    myRadius(node.getRadius()) {
+    myRadius(node.getRadius()) 
+{
+    if (node.getEdges().size() > 4 && !NBNodeTypeComputer::isRailwayNode(&node)) {
+        EXT = 50;
+    } else {
+        EXT = 100;
+    }
 }
 
 
@@ -61,7 +67,7 @@ NBNodeShapeComputer::~NBNodeShapeComputer() {}
 
 
 const PositionVector
-NBNodeShapeComputer::compute() {
+NBNodeShapeComputer::compute(bool forceSmall) {
 #ifdef DEBUG_NODE_SHAPE
     if (DEBUGCOND) {
         // annotate edges edges to make their ordering visible
@@ -74,7 +80,7 @@ NBNodeShapeComputer::compute() {
 #endif
     // check whether the node is a dead end node or a node where only turning is possible
     //  in this case, we will use "computeNodeShapeSmall"
-    if (myNode.getEdges().size() == 1) {
+    if (myNode.getEdges().size() == 1 || forceSmall) {
         return computeNodeShapeSmall();
     }
     if (myNode.getEdges().size() == 2 && myNode.getIncomingEdges().size() == 1) {
@@ -93,7 +99,7 @@ NBNodeShapeComputer::compute() {
 
 
 void
-computeSameEnd(PositionVector& l1, PositionVector& l2) {
+NBNodeShapeComputer::computeSameEnd(PositionVector& l1, PositionVector& l2) {
     assert(l1[0].distanceTo2D(l1[1]) >= EXT);
     assert(l2[0].distanceTo2D(l2[1]) >= EXT);
     PositionVector tmp;
@@ -126,7 +132,7 @@ NBNodeShapeComputer::computeNodeShapeDefault(bool simpleContinuation) {
     const double defaultRadius = getDefaultRadius(oc);
     const bool useDefaultRadius = myNode.getRadius() == NBNode::UNSPECIFIED_RADIUS || myNode.getRadius() == defaultRadius;
     myRadius = (useDefaultRadius ? defaultRadius : myNode.getRadius());
-    const double smallRadius = oc.getFloat("junctions.small-radius");
+    double smallRadius = useDefaultRadius ? oc.getFloat("junctions.small-radius") : myRadius;
     const int cornerDetail = oc.getInt("junctions.corner-detail");
     const double sCurveStretch = oc.getFloat("junctions.scurve-stretch");
     const bool rectangularCut = oc.getBool("rectangular-lane-cut");
@@ -221,6 +227,10 @@ NBNodeShapeComputer::computeNodeShapeDefault(bool simpleContinuation) {
                               geomsCCW[*i].nearest_offset_to_point2D(p),
                               geomsCW[*i].nearest_offset_to_point2D(p));
             if (dist < 0) {
+                if (isRailway((*i)->getPermissions())) {
+                    // better not mess up bidi geometries
+                    return PositionVector();
+                }
                 // ok, we have the problem that even the extrapolated geometry
                 //  does not reach the point
                 // in this case, the geometry has to be extenden... too bad ...
@@ -261,6 +271,9 @@ NBNodeShapeComputer::computeNodeShapeDefault(bool simpleContinuation) {
                         }
                         radius2 = MAX2(radius2, width / 2);
                     }
+                    if (!useDefaultRadius) {
+                        radius2 = MAX2(radius2, myRadius);
+                    }
                     dist += radius2;
 #ifdef DEBUG_NODE_SHAPE
                     if (DEBUGCOND) {
@@ -434,7 +447,7 @@ NBNodeShapeComputer::computeNodeShapeDefault(bool simpleContinuation) {
             offset = MAX2(EXT - advanceStopLine, offset);
         }
         if (offset == -1) {
-            WRITE_WARNING("Fixing offset for edge '" + (*i)->getID() + "' at node '" + myNode.getID() + ".");
+            WRITE_WARNINGF(TL("Fixing offset for edge '%' at node '%."), (*i)->getID(), myNode.getID());
             offset = (double) - .1;
         }
         Position p = ccwBound.positionAtOffset2D(offset);
@@ -615,9 +628,13 @@ NBNodeShapeComputer::computeEdgeBoundaries(const EdgeVector& edges,
         if (geomsCW[edge].length2D() < NUMERICAL_EPS) {
             geomsCW[edge] = edge->getGeometry();
         }
-        // extend the boundary by extrapolating it by EXT m
+        // cut off all parts beyond EXT to avoid issues with curved-back roads
+        geomsCCW[edge] = geomsCCW[edge].getSubpart2D(0, MAX2(EXT, edge->getTotalWidth()));
+        geomsCW[edge] = geomsCW[edge].getSubpart2D(0, MAX2(EXT, edge->getTotalWidth()));
+        // extend the boundary by extrapolating it by EXT m towards the junction
         geomsCCW[edge].extrapolate2D(EXT, true);
         geomsCW[edge].extrapolate2D(EXT, true);
+        // ensure minimum length by extending it away from the junction
         geomsCCW[edge].extrapolate(EXT2, false, true);
         geomsCW[edge].extrapolate(EXT2, false, true);
     }
@@ -652,7 +669,7 @@ NBNodeShapeComputer::joinSameDirectionEdges(const EdgeVector& edges, std::map<NB
         const bool ambiguousGeometry = ((angleDiff > 0 && angleDiffFurther < 0) || (angleDiff < 0 && angleDiffFurther > 0));
         //if (ambiguousGeometry) {
         //    @todo: this warning would be helpful in many cases. However, if angle and angleFurther jump between 179 and -179 it is misleading
-        //    WRITE_WARNING("Ambiguous angles at junction '" + myNode.getID() + "' for edges '" + (*i)->getID() + "' and '" + (*j)->getID() + "'.");
+        //    WRITE_WARNINGF(TL("Ambiguous angles at junction '%' for edges '%' and '%'."), myNode.getID(), (*i)->getID(), (*j)->getID());
         //}
 #ifdef DEBUG_NODE_SHAPE
         if (DEBUGCOND) {
