@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2022 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2023 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -30,6 +30,7 @@
 #include <netedit/frames/data/GNEEdgeDataFrame.h>
 #include <netedit/frames/data/GNEEdgeRelDataFrame.h>
 #include <netedit/frames/data/GNETAZRelDataFrame.h>
+#include <netedit/frames/data/GNEMeanDataFrame.h>
 #include <netedit/frames/demand/GNEContainerFrame.h>
 #include <netedit/frames/demand/GNEContainerPlanFrame.h>
 #include <netedit/frames/demand/GNEPersonFrame.h>
@@ -38,6 +39,7 @@
 #include <netedit/frames/demand/GNEStopFrame.h>
 #include <netedit/frames/demand/GNEVehicleFrame.h>
 #include <netedit/frames/demand/GNETypeFrame.h>
+#include <netedit/frames/demand/GNETypeDistributionFrame.h>
 #include <netedit/frames/network/GNEAdditionalFrame.h>
 #include <netedit/frames/network/GNEConnectorFrame.h>
 #include <netedit/frames/network/GNECreateEdgeFrame.h>
@@ -63,17 +65,17 @@
 
 FXDEFMAP(GNEViewParent) GNEViewParentMap[] = {
     FXMAPFUNC(SEL_COMMAND,  MID_MAKESNAPSHOT,                       GNEViewParent::onCmdMakeSnapshot),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEJUNCTION,                     GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEEDGE,                         GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEWALKINGAREA,                  GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEVEHICLE,                      GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEPERSON,                       GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEROUTE,                        GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATESTOP,                         GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATETLS,                          GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEADD,                          GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEPOI,                          GNEViewParent::onCmdLocate),
-    FXMAPFUNC(SEL_COMMAND,  MID_LOCATEPOLY,                         GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_J_LOCATEJUNCTION,      GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_E_LOCATEEDGE,          GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_W_LOCATEWALKINGAREA,   GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_V_LOCATEVEHICLE,       GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_P_LOCATEPERSON,        GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_R_LOCATEROUTE,         GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_S_LOCATESTOP,          GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_T_LOCATETLS,           GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_A_LOCATEADDITIONAL,    GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_O_LOCATEPOI,           GNEViewParent::onCmdLocate),
+    FXMAPFUNC(SEL_COMMAND,  MID_HOTKEY_SHIFT_L_LOCATEPOLY,          GNEViewParent::onCmdLocate),
     FXMAPFUNC(SEL_COMMAND,  FXMDIChild::ID_MDI_MENUCLOSE,           GNEViewParent::onCmdClose),
     FXMAPFUNC(SEL_CHANGED,  MID_GNE_VIEWPARENT_FRAMEAREAWIDTH,      GNEViewParent::onCmdUpdateFrameAreaWidth),
 };
@@ -87,27 +89,30 @@ FXIMPLEMENT(GNEViewParent, GUIGlChildWindow, GNEViewParentMap, ARRAYNUMBER(GNEVi
 // ===========================================================================
 
 GNEViewParent::GNEViewParent(FXMDIClient* p, FXMDIMenu* mdimenu, const FXString& name, GNEApplicationWindow* parentWindow,
-                             FXGLCanvas* share, GNENet* net, const bool newNet, GNEUndoList* undoList, FXIcon* ic, FXuint opts, FXint x, FXint y, FXint w, FXint h) :
+                             FXGLCanvas* share, GNENet* net, GNEUndoList* undoList, FXIcon* ic, FXuint opts, FXint x, FXint y, FXint w, FXint h) :
     GUIGlChildWindow(p, parentWindow, mdimenu, name, parentWindow->getToolbarsGrip().navigation, ic, opts, x, y, w, h),
     myGNEAppWindows(parentWindow) {
     // Add child to parent
-    myParent->addGLChild(this);
+    myGUIMainWindowParent->addGLChild(this);
 
     // Create Vertical separator
     new FXVerticalSeparator(myGripNavigationToolbar, GUIDesignVerticalSeparator);
 
     // Create undo/redo buttons
     myUndoButton = new MFXButtonTooltip(myGripNavigationToolbar, myGNEAppWindows->getStaticTooltipMenu(),
-                                        "\tUndo\tUndo the last change. (Ctrl+Z)", GUIIconSubSys::getIcon(GUIIcon::UNDO), parentWindow, MID_HOTKEY_CTRL_Z_UNDO, GUIDesignButtonToolbar);
+                                        std::string("\t") + TL("Undo") + std::string("\t") + TL("Undo the last change. (Ctrl+Z)"),
+                                        GUIIconSubSys::getIcon(GUIIcon::UNDO), parentWindow, MID_HOTKEY_CTRL_Z_UNDO, GUIDesignButtonToolbar);
     myRedoButton = new MFXButtonTooltip(myGripNavigationToolbar, myGNEAppWindows->getStaticTooltipMenu(),
-                                        "\tRedo\tRedo the last change. (Ctrl+Y)", GUIIconSubSys::getIcon(GUIIcon::REDO), parentWindow, MID_HOTKEY_CTRL_Y_REDO, GUIDesignButtonToolbar);
+                                        std::string("\t") + TL("Redo") + std::string("\t") + TL("Redo the last change. (Ctrl+Y)"),
+                                        GUIIconSubSys::getIcon(GUIIcon::REDO), parentWindow, MID_HOTKEY_CTRL_Y_REDO, GUIDesignButtonToolbar);
 
     // Create Vertical separator
     new FXVerticalSeparator(myGripNavigationToolbar, GUIDesignVerticalSeparator);
 
     // create compute path manager button
     myComputePathManagerButton = new MFXButtonTooltip(myGripNavigationToolbar, myGNEAppWindows->getStaticTooltipMenu(),
-            "\tCompute path manager\tCompute path manager", GUIIconSubSys::getIcon(GUIIcon::COMPUTEPATHMANAGER), parentWindow, MID_GNE_TOOLBAREDIT_COMPUTEPATHMANAGER, GUIDesignButtonToolbar);
+            std::string("\t") + TL("Compute path manager") + std::string("\t") + TL("Compute path manager."),
+            GUIIconSubSys::getIcon(GUIIcon::COMPUTEPATHMANAGER), parentWindow, MID_GNE_TOOLBAREDIT_COMPUTEPATHMANAGER, GUIDesignButtonToolbar);
 
     // Create Frame Splitter
     myFramesSplitter = new FXSplitter(myChildWindowContentFrame, this, MID_GNE_VIEWPARENT_FRAMEAREAWIDTH, GUIDesignSplitter | SPLITTER_HORIZONTAL);
@@ -122,7 +127,7 @@ GNEViewParent::GNEViewParent(FXMDIClient* p, FXMDIMenu* mdimenu, const FXString&
     FXComposite* tmp = new FXComposite(this);
 
     // Create view net
-    GNEViewNet* viewNet = new GNEViewNet(tmp, myViewArea, *myParent, this, net, newNet, undoList, myParent->getGLVisual(), share);
+    GNEViewNet* viewNet = new GNEViewNet(tmp, myViewArea, *myGUIMainWindowParent, this, net, undoList, myGUIMainWindowParent->getGLVisual(), share);
 
     // show toolbar grips
     myGNEAppWindows->getToolbarsGrip().buildMenuToolbarsGrip();
@@ -157,7 +162,7 @@ GNEViewParent::~GNEViewParent() {
     // delete toolbar grips
     myGNEAppWindows->getToolbarsGrip().destroyParentToolbarsGrips();
     // Remove child before remove
-    myParent->removeGLChild(this);
+    myGUIMainWindowParent->removeGLChild(this);
 }
 
 
@@ -278,7 +283,13 @@ GNEViewParent::getVehicleFrame() const {
 
 GNETypeFrame*
 GNEViewParent::getTypeFrame() const {
-    return myDemandFrames.vehicleTypeFrame;
+    return myDemandFrames.typeFrame;
+}
+
+
+GNETypeDistributionFrame*
+GNEViewParent::getTypeDistributionFrame() const {
+    return myDemandFrames.typeDistributionFrame;
 }
 
 
@@ -330,6 +341,12 @@ GNEViewParent::getTAZRelDataFrame() const {
 }
 
 
+GNEMeanDataFrame*
+GNEViewParent::getMeanDataFrame() const {
+    return myDataFrames.meanDataFrame;
+}
+
+
 void
 GNEViewParent::showFramesArea() {
     const bool frameShown = myCommonFrames.isCommonFrameShown() ||
@@ -360,7 +377,7 @@ GNEViewParent::hideFramesArea() {
 
 GUIMainWindow*
 GNEViewParent::getGUIMainWindow() const {
-    return myParent;
+    return myGUIMainWindowParent;
 }
 
 
@@ -474,7 +491,7 @@ GNEViewParent::onCmdMakeSnapshot(FXObject*, FXSelector, void*) {
 
 long
 GNEViewParent::onCmdClose(FXObject*, FXSelector /* sel */, void*) {
-    myParent->handle(this, FXSEL(SEL_COMMAND, MID_HOTKEY_CTRL_W_CLOSESIMULATION), nullptr);
+    myGUIMainWindowParent->handle(this, FXSEL(SEL_COMMAND, MID_HOTKEY_CTRL_W_CLOSESIMULATION), nullptr);
     return 1;
 }
 
@@ -490,43 +507,45 @@ GNEViewParent::onCmdLocate(FXObject*, FXSelector sel, void*) {
         GNEDialogACChooser** chooserLoc = nullptr;
         std::string locateTitle;
         switch (messageId) {
-            case MID_LOCATEJUNCTION:
+            case MID_HOTKEY_SHIFT_J_LOCATEJUNCTION:
                 chooserLoc = &myACChoosers.ACChooserJunction;
-                locateTitle = "Junction";
+                locateTitle = TL("Junction Chooser");
                 ACsToLocate.reserve(viewNet->getNet()->getAttributeCarriers()->getJunctions().size());
                 for (const auto& junction : viewNet->getNet()->getAttributeCarriers()->getJunctions()) {
                     ACsToLocate.push_back(junction.second);
                 }
                 break;
-            case MID_LOCATEEDGE:
+            case MID_HOTKEY_SHIFT_E_LOCATEEDGE:
                 chooserLoc = &myACChoosers.ACChooserEdges;
-                locateTitle = "Edge";
+                locateTitle = TL("Edge Chooser");
                 ACsToLocate.reserve(viewNet->getNet()->getAttributeCarriers()->getEdges().size());
                 for (const auto& edge : viewNet->getNet()->getAttributeCarriers()->getEdges()) {
                     ACsToLocate.push_back(edge.second);
                 }
                 break;
-            case MID_LOCATEWALKINGAREA:
+            case MID_HOTKEY_SHIFT_W_LOCATEWALKINGAREA:
                 chooserLoc = &myACChoosers.ACChooserWalkingAreas;
-                locateTitle = "WalkingArea";
+                locateTitle = TL("WalkingArea Chooser");
                 ACsToLocate.reserve(viewNet->getNet()->getAttributeCarriers()->getWalkingAreas().size());
                 for (const auto& walkingArea : viewNet->getNet()->getAttributeCarriers()->getWalkingAreas()) {
                     ACsToLocate.push_back(walkingArea);
                 }
                 break;
-            case MID_LOCATEVEHICLE: {
+            case MID_HOTKEY_SHIFT_V_LOCATEVEHICLE: {
                 chooserLoc = &myACChoosers.ACChooserVehicles;
-                locateTitle = "Vehicle";
+                locateTitle = TL("Vehicle Chooser");
                 const auto demandElements = viewNet->getNet()->getAttributeCarriers()->getDemandElements();
                 // reserve memory
                 ACsToLocate.reserve(demandElements.at(SUMO_TAG_VEHICLE).size() +
                                     demandElements.at(SUMO_TAG_TRIP).size() +
                                     demandElements.at(GNE_TAG_VEHICLE_WITHROUTE).size() +
                                     demandElements.at(GNE_TAG_TRIP_JUNCTIONS).size() +
+                                    demandElements.at(GNE_TAG_TRIP_TAZS).size() +
                                     demandElements.at(SUMO_TAG_FLOW).size() +
                                     demandElements.at(GNE_TAG_FLOW_ROUTE).size() +
                                     demandElements.at(GNE_TAG_FLOW_WITHROUTE).size() +
-                                    demandElements.at(GNE_TAG_FLOW_JUNCTIONS).size());
+                                    demandElements.at(GNE_TAG_FLOW_JUNCTIONS).size() + 
+                                    demandElements.at(GNE_TAG_FLOW_TAZS).size());
                 // fill ACsToLocate with vehicles,...
                 for (const auto& vehicle : demandElements.at(SUMO_TAG_VEHICLE)) {
                     ACsToLocate.push_back(vehicle);
@@ -536,12 +555,16 @@ GNEViewParent::onCmdLocate(FXObject*, FXSelector sel, void*) {
                     ACsToLocate.push_back(trip);
                 }
                 // ...vehicles with embedded routes,...
-                for (const auto& trip : demandElements.at(GNE_TAG_VEHICLE_WITHROUTE)) {
-                    ACsToLocate.push_back(trip);
+                for (const auto& tripEmbedded : demandElements.at(GNE_TAG_VEHICLE_WITHROUTE)) {
+                    ACsToLocate.push_back(tripEmbedded);
                 }
                 // ...trips over junctions,...
-                for (const auto& trip : demandElements.at(GNE_TAG_TRIP_JUNCTIONS)) {
-                    ACsToLocate.push_back(trip);
+                for (const auto& tripJunction : demandElements.at(GNE_TAG_TRIP_JUNCTIONS)) {
+                    ACsToLocate.push_back(tripJunction);
+                }
+                // ...trips over TAZs,...
+                for (const auto& tripTAZ : demandElements.at(GNE_TAG_TRIP_TAZS)) {
+                    ACsToLocate.push_back(tripTAZ);
                 }
                 // ...flows,...
                 for (const auto& flow : demandElements.at(SUMO_TAG_FLOW)) {
@@ -552,18 +575,22 @@ GNEViewParent::onCmdLocate(FXObject*, FXSelector sel, void*) {
                     ACsToLocate.push_back(flowRoute);
                 }
                 // ...flows with embedded routes...
-                for (const auto& flowRoute : demandElements.at(GNE_TAG_FLOW_WITHROUTE)) {
-                    ACsToLocate.push_back(flowRoute);
+                for (const auto& flowRouteEmbedded : demandElements.at(GNE_TAG_FLOW_WITHROUTE)) {
+                    ACsToLocate.push_back(flowRouteEmbedded);
                 }
                 // ... and flows over junctions.
-                for (const auto& flowRoute : demandElements.at(GNE_TAG_FLOW_JUNCTIONS)) {
-                    ACsToLocate.push_back(flowRoute);
+                for (const auto& flowJunction : demandElements.at(GNE_TAG_FLOW_JUNCTIONS)) {
+                    ACsToLocate.push_back(flowJunction);
+                }
+                // ... and flows over TAZs.
+                for (const auto& flowTAZ : demandElements.at(GNE_TAG_FLOW_TAZS)) {
+                    ACsToLocate.push_back(flowTAZ);
                 }
                 break;
             }
-            case MID_LOCATEPERSON:
+            case MID_HOTKEY_SHIFT_P_LOCATEPERSON:
                 chooserLoc = &myACChoosers.ACChooserPersons;
-                locateTitle = "Person";
+                locateTitle = TL("Person Chooser");
                 ACsToLocate.reserve(viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_PERSON).size() +
                                     viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_PERSONFLOW).size());
                 // fill ACsToLocate with persons
@@ -575,48 +602,53 @@ GNEViewParent::onCmdLocate(FXObject*, FXSelector sel, void*) {
                     ACsToLocate.push_back(personFlow);
                 }
                 break;
-            case MID_LOCATEROUTE:
+            case MID_HOTKEY_SHIFT_R_LOCATEROUTE:
                 chooserLoc = &myACChoosers.ACChooserRoutes;
-                locateTitle = "Route";
+                locateTitle = TL("Route Chooser");
                 ACsToLocate.reserve(viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_ROUTE).size());
                 for (const auto& route : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_ROUTE)) {
                     ACsToLocate.push_back(route);
                 }
                 break;
-            case MID_LOCATESTOP: {
+            case MID_HOTKEY_SHIFT_S_LOCATESTOP: {
                 chooserLoc = &myACChoosers.ACChooserStops;
-                locateTitle = "Stop";
+                locateTitle = TL("Stop Chooser");
                 // reserve memory
-                ACsToLocate.reserve(viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_LANE).size() +
-                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_BUSSTOP).size() +
-                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_CONTAINERSTOP).size() +
-                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_CHARGINGSTATION).size() +
-                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_PARKINGAREA).size());
+                ACsToLocate.reserve(viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_LANE).size() +
+                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_BUSSTOP).size() +
+                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_TRAINSTOP).size() +
+                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_CONTAINERSTOP).size() +
+                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_CHARGINGSTATION).size() +
+                                    viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_PARKINGAREA).size());
                 // fill ACsToLocate with stop over lanes
-                for (const auto& stopLane : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_LANE)) {
+                for (const auto& stopLane : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_LANE)) {
                     ACsToLocate.push_back(stopLane);
                 }
-                // fill ACsToLocate with stop over busstops
-                for (const auto& stopBusStop : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_BUSSTOP)) {
+                // fill ACsToLocate with stop over bus stops
+                for (const auto& stopBusStop : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_BUSSTOP)) {
                     ACsToLocate.push_back(stopBusStop);
                 }
+                // fill ACsToLocate with stop over train stops
+                for (const auto& stopTrainStop : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_TRAINSTOP)) {
+                    ACsToLocate.push_back(stopTrainStop);
+                }
                 // fill ACsToLocate with stop over container stops
-                for (const auto& stopContainerStop : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_CONTAINERSTOP)) {
+                for (const auto& stopContainerStop : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_CONTAINERSTOP)) {
                     ACsToLocate.push_back(stopContainerStop);
                 }
                 // fill ACsToLocate with stop over charging stations
-                for (const auto& stopChargingStation : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_CHARGINGSTATION)) {
+                for (const auto& stopChargingStation : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_CHARGINGSTATION)) {
                     ACsToLocate.push_back(stopChargingStation);
                 }
                 // fill ACsToLocate with stop over parking areas
-                for (const auto& stopParkingArea : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(SUMO_TAG_STOP_PARKINGAREA)) {
+                for (const auto& stopParkingArea : viewNet->getNet()->getAttributeCarriers()->getDemandElements().at(GNE_TAG_STOP_PARKINGAREA)) {
                     ACsToLocate.push_back(stopParkingArea);
                 }
                 break;
             }
-            case MID_LOCATETLS:
+            case MID_HOTKEY_SHIFT_T_LOCATETLS:
                 chooserLoc = &myACChoosers.ACChooserTLS;
-                locateTitle = "TLS";
+                locateTitle = TL("TLS Chooser");
                 // fill ACsToLocate with junctions that haven TLS
                 ACsToLocate.reserve(viewNet->getNet()->getAttributeCarriers()->getJunctions().size());
                 for (const auto& junction : viewNet->getNet()->getAttributeCarriers()->getJunctions()) {
@@ -625,9 +657,9 @@ GNEViewParent::onCmdLocate(FXObject*, FXSelector sel, void*) {
                     }
                 }
                 break;
-            case MID_LOCATEADD:
+            case MID_HOTKEY_SHIFT_A_LOCATEADDITIONAL:
                 chooserLoc = &myACChoosers.ACChooserAdditional;
-                locateTitle = "Additional";
+                locateTitle = TL("Additional Chooser");
                 for (const auto& additionalTag : viewNet->getNet()->getAttributeCarriers()->getAdditionals()) {
                     if (additionalTag.first == SUMO_TAG_POI
                             || additionalTag.first == GNE_TAG_POILANE
@@ -640,9 +672,9 @@ GNEViewParent::onCmdLocate(FXObject*, FXSelector sel, void*) {
                     }
                 }
                 break;
-            case MID_LOCATEPOI:
+            case MID_HOTKEY_SHIFT_O_LOCATEPOI:
                 chooserLoc = &myACChoosers.ACChooserPOI;
-                locateTitle = "POI";
+                locateTitle = TL("POI Chooser");
                 // fill ACsToLocate with POIs
                 for (const auto& POI : viewNet->getNet()->getAttributeCarriers()->getAdditionals().at(SUMO_TAG_POI)) {
                     ACsToLocate.push_back(POI);
@@ -654,9 +686,9 @@ GNEViewParent::onCmdLocate(FXObject*, FXSelector sel, void*) {
                     ACsToLocate.push_back(POI);
                 }
                 break;
-            case MID_LOCATEPOLY:
+            case MID_HOTKEY_SHIFT_L_LOCATEPOLY:
                 chooserLoc = &myACChoosers.ACChooserPolygon;
-                locateTitle = "Poly";
+                locateTitle = TL("Poly Chooser");
                 // fill ACsToLocate with polys and TAZs (because share namespae)
                 for (const auto& polygon : viewNet->getNet()->getAttributeCarriers()->getAdditionals().at(SUMO_TAG_POLY)) {
                     ACsToLocate.push_back(polygon);
@@ -675,7 +707,7 @@ GNEViewParent::onCmdLocate(FXObject*, FXSelector sel, void*) {
             chooser->setFocus();
             chooser->raise();
         } else {
-            GNEDialogACChooser* chooser = new GNEDialogACChooser(this, messageId, GUIIconSubSys::getIcon(GUIIcon::LOCATEJUNCTION), locateTitle + " Chooser", ACsToLocate);
+            GNEDialogACChooser* chooser = new GNEDialogACChooser(this, messageId, GUIIconSubSys::getIcon(GUIIcon::LOCATEJUNCTION), locateTitle, ACsToLocate);
             *chooserLoc = chooser;
         }
         // update locator popup
@@ -902,15 +934,7 @@ GNEViewParent::NetworkFrames::getCurrentShownFrame() const {
 // GNEViewParent::DemandFrames - methods
 // ---------------------------------------------------------------------------
 
-GNEViewParent::DemandFrames::DemandFrames() :
-    routeFrame(nullptr),
-    vehicleFrame(nullptr),
-    vehicleTypeFrame(nullptr),
-    stopFrame(nullptr),
-    personFrame(nullptr),
-    personPlanFrame(nullptr),
-    containerFrame(nullptr),
-    containerPlanFrame(nullptr) {
+GNEViewParent::DemandFrames::DemandFrames() {
 }
 
 
@@ -918,7 +942,8 @@ void
 GNEViewParent::DemandFrames::buildDemandFrames(GNEViewParent* viewParent, GNEViewNet* viewNet) {
     routeFrame = new GNERouteFrame(viewParent, viewNet);
     vehicleFrame = new GNEVehicleFrame(viewParent, viewNet);
-    vehicleTypeFrame = new GNETypeFrame(viewParent, viewNet);
+    typeFrame = new GNETypeFrame(viewParent, viewNet);
+    typeDistributionFrame = new GNETypeDistributionFrame(viewParent, viewNet);
     stopFrame = new GNEStopFrame(viewParent, viewNet);
     personFrame = new GNEPersonFrame(viewParent, viewNet);
     personPlanFrame = new GNEPersonPlanFrame(viewParent, viewNet);
@@ -931,7 +956,8 @@ void
 GNEViewParent::DemandFrames::hideDemandFrames() {
     routeFrame->hide();
     vehicleFrame->hide();
-    vehicleTypeFrame->hide();
+    typeFrame->hide();
+    typeDistributionFrame->hide();
     stopFrame->hide();
     personFrame->hide();
     personPlanFrame->hide();
@@ -945,7 +971,8 @@ GNEViewParent::DemandFrames::setDemandFramesWidth(int frameWidth) {
     // set width in all frames
     routeFrame->setFrameWidth(frameWidth);
     vehicleFrame->setFrameWidth(frameWidth);
-    vehicleTypeFrame->setFrameWidth(frameWidth);
+    typeFrame->setFrameWidth(frameWidth);
+    typeDistributionFrame->setFrameWidth(frameWidth);
     stopFrame->setFrameWidth(frameWidth);
     personFrame->setFrameWidth(frameWidth);
     personPlanFrame->setFrameWidth(frameWidth);
@@ -961,7 +988,9 @@ GNEViewParent::DemandFrames::isDemandFrameShown() const {
         return true;
     } else if (vehicleFrame->shown()) {
         return true;
-    } else if (vehicleTypeFrame->shown()) {
+    } else if (typeFrame->shown()) {
+        return true;
+    } else if (typeDistributionFrame->shown()) {
         return true;
     } else if (stopFrame->shown()) {
         return true;
@@ -986,8 +1015,10 @@ GNEViewParent::DemandFrames::getCurrentShownFrame() const {
         return routeFrame;
     } else if (vehicleFrame->shown()) {
         return vehicleFrame;
-    } else if (vehicleTypeFrame->shown()) {
-        return vehicleTypeFrame;
+    } else if (typeFrame->shown()) {
+        return typeFrame;
+    } else if (typeDistributionFrame->shown()) {
+        return typeDistributionFrame;
     } else if (stopFrame->shown()) {
         return stopFrame;
     } else if (personFrame->shown()) {
@@ -1007,10 +1038,7 @@ GNEViewParent::DemandFrames::getCurrentShownFrame() const {
 // GNEViewParent::DataFrames - methods
 // ---------------------------------------------------------------------------
 
-GNEViewParent::DataFrames::DataFrames() :
-    edgeDataFrame(nullptr),
-    edgeRelDataFrame(nullptr),
-    TAZRelDataFrame(nullptr) {
+GNEViewParent::DataFrames::DataFrames() {
 }
 
 
@@ -1019,6 +1047,7 @@ GNEViewParent::DataFrames::buildDataFrames(GNEViewParent* viewParent, GNEViewNet
     edgeDataFrame = new GNEEdgeDataFrame(viewParent, viewNet);
     edgeRelDataFrame = new GNEEdgeRelDataFrame(viewParent, viewNet);
     TAZRelDataFrame = new GNETAZRelDataFrame(viewParent, viewNet);
+    meanDataFrame = new GNEMeanDataFrame(viewParent, viewNet);
 }
 
 
@@ -1027,6 +1056,7 @@ GNEViewParent::DataFrames::hideDataFrames() {
     edgeDataFrame->hide();
     edgeRelDataFrame->hide();
     TAZRelDataFrame->hide();
+    meanDataFrame->hide();
 }
 
 
@@ -1036,6 +1066,7 @@ GNEViewParent::DataFrames::setDataFramesWidth(int frameWidth) {
     edgeDataFrame->setFrameWidth(frameWidth);
     edgeRelDataFrame->setFrameWidth(frameWidth);
     TAZRelDataFrame->setFrameWidth(frameWidth);
+    meanDataFrame->setFrameWidth(frameWidth);
 }
 
 
@@ -1048,6 +1079,8 @@ GNEViewParent::DataFrames::isDataFrameShown() const {
         return true;
     } else if (TAZRelDataFrame->shown()) {
         return true;
+    } else if (meanDataFrame->shown()) {
+        return true;
     } else {
         return false;
     }
@@ -1063,6 +1096,8 @@ GNEViewParent::DataFrames::getCurrentShownFrame() const {
         return edgeRelDataFrame;
     } else if (TAZRelDataFrame->shown()) {
         return TAZRelDataFrame;
+    } else if (meanDataFrame->shown()) {
+        return meanDataFrame;
     } else {
         return nullptr;
     }
